import React from 'react';
import warning from 'tiny-warning';
import { isResolved } from './ResolverUtils';
import { jsx as _jsx } from "react/jsx-runtime";
export default function createElements(routeMatches, Components, matchData) {
  return routeMatches.map((match, i) => {
    const {
      router,
      route
    } = match;
    const Component = Components[i];
    const data = matchData[i];
    const isComponentResolved = isResolved(Component);
    const areDataResolved = isResolved(data);

    if (route.render) {
      // Perhaps undefined here would be more correct for "not ready", but
      // Relay uses null in RelayReadyStateRenderer, so let's follow that
      // convention.
      return route.render({
        match,
        Component: isComponentResolved ? Component : null,
        props: areDataResolved ? {
          match,
          router,
          data
        } : null,
        data: areDataResolved ? data : null
      });
    }

    if (!isComponentResolved || !areDataResolved) {
      // Can't render.
      return undefined;
    }

    if (!Component) {
      // Note this check would be wrong on potentially unresolved data.
      process.env.NODE_ENV !== "production" ? warning(data === undefined, 'Route %s with data has no render method or component.', i) : void 0; // Nothing to render.

      return null;
    }

    return /*#__PURE__*/_jsx(Component, {
      match: match,
      router: router,
      data: data
    });
  });
}