import _awaitAsyncGenerator from "@babel/runtime/helpers/esm/awaitAsyncGenerator";
import _wrapAsyncGenerator from "@babel/runtime/helpers/esm/wrapAsyncGenerator";
import isPromise from 'is-promise';
import { accumulateRouteValues, checkResolved, getComponents, getRouteMatches, getRouteValue, isResolved } from './ResolverUtils';
import createElements from './createElements';

function getRouteGetData(route) {
  return route.getData;
}

function getRouteData(route) {
  return route.data;
}

export default {
  resolveElements(match) {
    var _this = this;

    return _wrapAsyncGenerator(function* () {
      const routeMatches = getRouteMatches(match);
      const Components = getComponents(routeMatches);

      const data = _this.getData(match, routeMatches);

      const earlyComponents = Components.some(isPromise) ? yield _awaitAsyncGenerator(Promise.all(Components.map(checkResolved))) : Components;
      const earlyData = data.some(isPromise) ? yield _awaitAsyncGenerator(Promise.all(data.map(checkResolved))) : data;
      let fetchedComponents;
      let fetchedData;

      if (!earlyComponents.every(isResolved) || !earlyData.every(isResolved)) {
        const pendingElements = createElements(routeMatches, earlyComponents, earlyData);
        yield pendingElements.every(element => element !== undefined) ? pendingElements : undefined;
        fetchedComponents = yield _awaitAsyncGenerator(Promise.all(Components));
        fetchedData = yield _awaitAsyncGenerator(Promise.all(data));
      } else {
        fetchedComponents = earlyComponents;
        fetchedData = earlyData;
      }

      yield createElements(routeMatches, fetchedComponents, fetchedData);
    })();
  },

  /**
   * Generate route data according to their getters, respecting the order of
   * promises per the `defer` flag on routes.
   */
  getData(match, routeMatches) {
    return accumulateRouteValues(routeMatches, match.routeIndices, ({
      ancestorRouteData,
      prevParentPromise
    }, routeMatch) => {
      // For a deferred route, the parent promise is the previous promise.
      // Otherwise, it's the previous parent promise.
      const parentPromise = routeMatch.route.defer ? Promise.all(ancestorRouteData) : prevParentPromise; // If there is a parent promise, execute after it resolves.

      const routeData = parentPromise ? parentPromise.then(() => getRouteValue(routeMatch, getRouteGetData, getRouteData)) : getRouteValue(routeMatch, getRouteGetData, getRouteData);
      return {
        routeData,
        ancestorRouteData: [...ancestorRouteData, routeData],
        prevParentPromise: parentPromise
      };
    }, {
      routeData: null,
      ancestorRouteData: [],
      prevParentPromise: null
    }).map(({
      routeData
    }) => routeData);
  }

};